@@ -42,6 +42,7 @@
 
 #undef G_DISABLE_DEPRECATED
 
+#include "galias.h"
 #include "glib.h"
 
 #include "giochannel.h"
@@ -186,7 +187,7 @@ g_io_channel_read (GIOChannel *channel,
     {
       if (bytes_read)
         *bytes_read = 0;
-      return G_IO_STATUS_NORMAL;
+      return G_IO_ERROR_NONE;
     }
 
   g_return_val_if_fail (buf != NULL, G_IO_ERROR_UNKNOWN);
@@ -706,7 +707,7 @@ G_CONST_RETURN gchar*
 g_io_channel_get_line_term (GIOChannel	*channel,
 			    gint        *length)
 {
-  g_return_val_if_fail (channel != NULL, 0);
+  g_return_val_if_fail (channel != NULL, NULL);
 
   if (length)
     *length = channel->line_term_len;
