@@ -39,6 +39,7 @@ G_BEGIN_DECLS
  */
 #define G_DIR_SEPARATOR '\\'
 #define G_DIR_SEPARATOR_S "\\"
+#define G_IS_DIR_SEPARATOR(c) ((c) == G_DIR_SEPARATOR || (c) == '/')
 #define G_SEARCHPATH_SEPARATOR ';'
 #define G_SEARCHPATH_SEPARATOR_S ";"
 
@@ -48,6 +49,7 @@ G_BEGIN_DECLS
 
 #define G_DIR_SEPARATOR '/'
 #define G_DIR_SEPARATOR_S "/"
+#define G_IS_DIR_SEPARATOR(c) ((c) == G_DIR_SEPARATOR)
 #define G_SEARCHPATH_SEPARATOR ':'
 #define G_SEARCHPATH_SEPARATOR_S ":"
 
@@ -113,6 +115,10 @@ G_BEGIN_DECLS
 
 /* Retrive static string info
  */
+#ifdef G_OS_WIN32
+#define g_get_home_dir g_get_home_dir_utf8
+#define g_get_tmp_dir g_get_tmp_dir_utf8
+#endif
 G_CONST_RETURN gchar* g_get_user_name        (void);
 G_CONST_RETURN gchar* g_get_real_name        (void);
 G_CONST_RETURN gchar* g_get_home_dir         (void);
@@ -125,10 +131,10 @@ void                  g_set_application_name (const gchar *application_name);
 G_CONST_RETURN gchar*    g_get_user_data_dir      (void);
 G_CONST_RETURN gchar*    g_get_user_config_dir    (void);
 G_CONST_RETURN gchar*    g_get_user_cache_dir     (void);
-G_CONST_RETURN gchar * G_CONST_RETURN * g_get_system_data_dirs   (void);
-G_CONST_RETURN gchar * G_CONST_RETURN * g_get_system_config_dirs (void);
+G_CONST_RETURN gchar* G_CONST_RETURN * g_get_system_data_dirs   (void);
+G_CONST_RETURN gchar* G_CONST_RETURN * g_get_system_config_dirs (void);
 
-G_CONST_RETURN gchar * G_CONST_RETURN * g_get_language_names (void);
+G_CONST_RETURN gchar* G_CONST_RETURN * g_get_language_names (void);
 
 typedef struct _GDebugKey	GDebugKey;
 struct _GDebugKey
@@ -169,6 +175,10 @@ G_CONST_RETURN gchar* g_basename           (const gchar *file_name);
 
 #endif /* G_DISABLE_DEPRECATED */
 
+#ifdef G_OS_WIN32
+#define g_get_current_dir g_get_current_dir_utf8
+#endif
+
 /* The returned strings are newly allocated with g_malloc() */
 gchar*                g_get_current_dir    (void);
 gchar*                g_path_get_basename  (const gchar *file_name);
