@@ -1,5 +1,5 @@
 #! /bin/sh
 
-cpp -P -DG_OS_UNIX ${srcdir:-.}/gobject.symbols | sed -e '/^$/d' | sort > expected-abi
+cpp -P -DG_OS_UNIX ${srcdir:-.}/gobject.symbols | sed -e '/^$/d' -e 's/ PRIVATE$//' | sort > expected-abi
 nm -D .libs/libgobject-2.0.so | grep " T " | cut -d ' ' -f 3 | sort > actual-abi
 diff -u expected-abi actual-abi && rm expected-abi actual-abi
