@@ -16,6 +16,7 @@
  * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA 02111-1307, USA.
  */
+#include	"gobjectalias.h"
 #include	"gobject.h"
 
 /*
@@ -107,6 +108,8 @@ static GQuark	            quark_weak_refs = 0;
 static GParamSpecPool      *pspec_pool = NULL;
 static GObjectNotifyContext property_notify_context = { 0, };
 static gulong	            gobject_signals[LAST_SIGNAL] = { 0, };
+G_LOCK_DEFINE_STATIC (construct_objects_lock);
+static GSList *construct_objects = NULL;
 
 
 /* --- functions --- */
@@ -142,7 +145,7 @@ debug_objects_atexit (void)
 #endif	/* G_ENABLE_DEBUG */
 
 void
-g_object_type_init (void)	/* sync with gtype.c */
+g_object_type_init (void)
 {
   static gboolean initialized = FALSE;
   static const GTypeFundamentalInfo finfo = {
@@ -253,7 +256,7 @@ g_object_do_class_init (GObjectClass *class)
   gobject_signals[NOTIFY] =
     g_signal_new ("notify",
 		  G_TYPE_FROM_CLASS (class),
-		  G_SIGNAL_RUN_FIRST | G_SIGNAL_NO_RECURSE | G_SIGNAL_DETAILED | G_SIGNAL_NO_HOOKS,
+		  G_SIGNAL_RUN_FIRST | G_SIGNAL_NO_RECURSE | G_SIGNAL_DETAILED | G_SIGNAL_NO_HOOKS | G_SIGNAL_ACTION,
 		  G_STRUCT_OFFSET (GObjectClass, notify),
 		  NULL, NULL,
 		  g_cclosure_marshal_VOID__PARAM,
@@ -414,7 +417,7 @@ g_object_class_override_property (GObjectClass *oclass,
   if (!overridden)
     {
       g_warning ("%s: Can't find property to override for '%s::%s'",
-		 G_STRLOC, G_OBJECT_CLASS_NAME (oclass), name);
+		 G_STRFUNC, G_OBJECT_CLASS_NAME (oclass), name);
       return;
     }
 
@@ -467,6 +470,11 @@ g_object_init (GObject *object)
   
   /* freeze object's notification queue, g_object_newv() preserves pairedness */
   g_object_notify_queue_freeze (object, &property_notify_context);
+
+  /* allow construct-only properties to be set */
+  G_LOCK (construct_objects_lock);
+  construct_objects = g_slist_prepend (construct_objects, object);
+  G_UNLOCK (construct_objects_lock);
   
 #ifdef	G_ENABLE_DEBUG
   IF_DEBUG (OBJECTS)
@@ -631,7 +639,7 @@ g_object_notify (GObject     *object,
 
   if (!pspec)
     g_warning ("%s: object class `%s' has no property named `%s'",
-	       G_STRLOC,
+	       G_STRFUNC,
 	       G_OBJECT_TYPE_NAME (object),
 	       property_name);
   else
@@ -656,8 +664,8 @@ g_object_thaw_notify (GObject *object)
   g_object_ref (object);
   nqueue = g_object_notify_queue_from_object (object, &property_notify_context);
   if (!nqueue || !nqueue->freeze_count)
-    g_warning (G_STRLOC ": property-changed notification for %s(%p) is not frozen",
-	       G_OBJECT_TYPE_NAME (object), object);
+    g_warning ("%s: property-changed notification for %s(%p) is not frozen",
+	       G_STRFUNC, G_OBJECT_TYPE_NAME (object), object);
   else
     g_object_notify_queue_thaw (object, nqueue);
   g_object_unref (object);
@@ -818,6 +826,16 @@ g_object_new (GType	   object_type,
   return object;
 }
 
+static gboolean
+object_in_construction (GObject *object)
+{
+  gboolean in_construction;
+  G_LOCK (construct_objects_lock);
+  in_construction = g_slist_find (construct_objects, object) != NULL;
+  G_UNLOCK (construct_objects_lock);
+  return in_construction;
+}
+
 gpointer
 g_object_newv (GType       object_type,
 	       guint       n_parameters,
@@ -826,7 +844,7 @@ g_object_newv (GType       object_type,
   GObjectConstructParam *cparams, *oparams;
   GObjectNotifyQueue *nqueue;
   GObject *object;
-  GObjectClass *class;
+  GObjectClass *class, *unref_class = NULL;
   GSList *slist;
   guint n_total_cparams = 0, n_cparams = 0, n_oparams = 0, n_cvalues;
   GValue *cvalues;
@@ -835,7 +853,9 @@ g_object_newv (GType       object_type,
 
   g_return_val_if_fail (G_TYPE_IS_OBJECT (object_type), NULL);
 
-  class = g_type_class_ref (object_type);
+  class = g_type_class_peek_static (object_type);
+  if (!class)
+    class = unref_class = g_type_class_ref (object_type);
   for (slist = class->construct_properties; slist; slist = slist->next)
     {
       clist = g_list_prepend (clist, slist->data);
@@ -855,7 +875,7 @@ g_object_newv (GType       object_type,
       if (!pspec)
 	{
 	  g_warning ("%s: object class `%s' has no property named `%s'",
-		     G_STRLOC,
+		     G_STRFUNC,
 		     g_type_name (object_type),
 		     parameters[i].name);
 	  continue;
@@ -863,7 +883,7 @@ g_object_newv (GType       object_type,
       if (!(pspec->flags & G_PARAM_WRITABLE))
 	{
 	  g_warning ("%s: property `%s' of object class `%s' is not writable",
-		     G_STRLOC,
+		     G_STRFUNC,
 		     pspec->name,
 		     g_type_name (object_type));
 	  continue;
@@ -874,8 +894,8 @@ g_object_newv (GType       object_type,
 
 	  if (!list)
 	    {
-	      g_warning (G_STRLOC ": construct property \"%s\" for object `%s' can't be set twice",
-			 pspec->name, g_type_name (object_type));
+	      g_warning ("%s: construct property \"%s\" for object `%s' can't be set twice",
+                         G_STRFUNC, pspec->name, g_type_name (object_type));
 	      continue;
 	    }
 	  cparams[n_cparams].pspec = pspec;
@@ -920,6 +940,9 @@ g_object_newv (GType       object_type,
 
   /* construct object from construction parameters */
   object = class->constructor (object_type, n_total_cparams, cparams);
+  G_LOCK (construct_objects_lock);
+  construct_objects = g_slist_remove (construct_objects, object);
+  G_UNLOCK (construct_objects_lock);
 
   /* free construction values */
   g_free (cparams);
@@ -936,7 +959,8 @@ g_object_newv (GType       object_type,
     object_set_property (object, oparams[i].pspec, oparams[i].value, nqueue);
   g_free (oparams);
 
-  g_type_class_unref (class);
+  if (unref_class)
+    g_type_class_unref (unref_class);
 
   /* release our own freeze count and handle notifications */
   g_object_notify_queue_thaw (object, nqueue);
@@ -974,7 +998,7 @@ g_object_new_valist (GType	  object_type,
       if (!pspec)
 	{
 	  g_warning ("%s: object class `%s' has no property named `%s'",
-		     G_STRLOC,
+		     G_STRFUNC,
 		     g_type_name (object_type),
 		     name);
 	  break;
@@ -990,12 +1014,9 @@ g_object_new_valist (GType	  object_type,
       G_VALUE_COLLECT (&params[n_params].value, var_args, 0, &error);
       if (error)
 	{
-	  g_warning ("%s: %s", G_STRLOC, error);
+	  g_warning ("%s: %s", G_STRFUNC, error);
 	  g_free (error);
-
-	  /* we purposely leak the value here, it might not be
-	   * in a sane state if an error condition occoured
-	   */
+          g_value_unset (&params[n_params].value);
 	  break;
 	}
       n_params++;
@@ -1022,7 +1043,7 @@ g_object_constructor (GType                  type,
 
   /* create object */
   object = (GObject*) g_type_create_instance (type);
-
+  
   /* set construction parameters */
   if (n_construct_properties)
     {
@@ -1074,7 +1095,7 @@ g_object_set_valist (GObject	 *object,
       if (!pspec)
 	{
 	  g_warning ("%s: object class `%s' has no property named `%s'",
-		     G_STRLOC,
+		     G_STRFUNC,
 		     G_OBJECT_TYPE_NAME (object),
 		     name);
 	  break;
@@ -1082,23 +1103,26 @@ g_object_set_valist (GObject	 *object,
       if (!(pspec->flags & G_PARAM_WRITABLE))
 	{
 	  g_warning ("%s: property `%s' of object class `%s' is not writable",
-		     G_STRLOC,
+		     G_STRFUNC,
 		     pspec->name,
 		     G_OBJECT_TYPE_NAME (object));
 	  break;
 	}
-      
+      if ((pspec->flags & G_PARAM_CONSTRUCT_ONLY) && !object_in_construction (object))
+        {
+          g_warning ("%s: construct property \"%s\" for object `%s' can't be set after construction",
+                     G_STRFUNC, pspec->name, G_OBJECT_TYPE_NAME (object));
+          break;
+        }
+
       g_value_init (&value, G_PARAM_SPEC_VALUE_TYPE (pspec));
       
       G_VALUE_COLLECT (&value, var_args, 0, &error);
       if (error)
 	{
-	  g_warning ("%s: %s", G_STRLOC, error);
+	  g_warning ("%s: %s", G_STRFUNC, error);
 	  g_free (error);
-	  
-	  /* we purposely leak the value here, it might not be
-	   * in a sane state if an error condition occoured
-	   */
+          g_value_unset (&value);
 	  break;
 	}
       
@@ -1138,7 +1162,7 @@ g_object_get_valist (GObject	 *object,
       if (!pspec)
 	{
 	  g_warning ("%s: object class `%s' has no property named `%s'",
-		     G_STRLOC,
+		     G_STRFUNC,
 		     G_OBJECT_TYPE_NAME (object),
 		     name);
 	  break;
@@ -1146,7 +1170,7 @@ g_object_get_valist (GObject	 *object,
       if (!(pspec->flags & G_PARAM_READABLE))
 	{
 	  g_warning ("%s: property `%s' of object class `%s' is not readable",
-		     G_STRLOC,
+		     G_STRFUNC,
 		     pspec->name,
 		     G_OBJECT_TYPE_NAME (object));
 	  break;
@@ -1159,7 +1183,7 @@ g_object_get_valist (GObject	 *object,
       G_VALUE_LCOPY (&value, var_args, 0, &error);
       if (error)
 	{
-	  g_warning ("%s: %s", G_STRLOC, error);
+	  g_warning ("%s: %s", G_STRFUNC, error);
 	  g_free (error);
 	  g_value_unset (&value);
 	  break;
@@ -1224,9 +1248,17 @@ g_object_set_property (GObject	    *object,
 				    TRUE);
   if (!pspec)
     g_warning ("%s: object class `%s' has no property named `%s'",
-	       G_STRLOC,
+	       G_STRFUNC,
 	       G_OBJECT_TYPE_NAME (object),
 	       property_name);
+  else if (!(pspec->flags & G_PARAM_WRITABLE))
+    g_warning ("%s: property `%s' of object class `%s' is not writable",
+               G_STRFUNC,
+               pspec->name,
+               G_OBJECT_TYPE_NAME (object));
+  else if ((pspec->flags & G_PARAM_CONSTRUCT_ONLY) && !object_in_construction (object))
+    g_warning ("%s: construct property \"%s\" for object `%s' can't be set after construction",
+               G_STRFUNC, pspec->name, G_OBJECT_TYPE_NAME (object));
   else
     object_set_property (object, pspec, value, nqueue);
   
@@ -1253,9 +1285,14 @@ g_object_get_property (GObject	   *object,
 				    TRUE);
   if (!pspec)
     g_warning ("%s: object class `%s' has no property named `%s'",
-	       G_STRLOC,
+	       G_STRFUNC,
 	       G_OBJECT_TYPE_NAME (object),
 	       property_name);
+  else if (!(pspec->flags & G_PARAM_READABLE))
+    g_warning ("%s: property `%s' of object class `%s' is not readable",
+               G_STRFUNC,
+               pspec->name,
+               G_OBJECT_TYPE_NAME (object));
   else
     {
       GValue *prop_value, tmp_value = { 0, };
@@ -1269,8 +1306,8 @@ g_object_get_property (GObject	   *object,
 	}
       else if (!g_value_type_transformable (G_PARAM_SPEC_VALUE_TYPE (pspec), G_VALUE_TYPE (value)))
 	{
-	  g_warning ("can't retrieve property `%s' of type `%s' as value of type `%s'",
-		     pspec->name,
+	  g_warning ("%s: can't retrieve property `%s' of type `%s' as value of type `%s'",
+		     G_STRFUNC, pspec->name,
 		     g_type_name (G_PARAM_SPEC_VALUE_TYPE (pspec)),
 		     G_VALUE_TYPE_NAME (value));
 	  g_object_unref (object);
@@ -1314,37 +1351,44 @@ g_object_connect (gpointer     _object,
 	sid = g_signal_connect_data (object, signal_spec + 8,
 				     callback, data, NULL,
 				     0);
-      else if (strncmp (signal_spec, "object_signal::", 15) == 0)
+      else if (strncmp (signal_spec, "object_signal::", 15) == 0 ||
+               strncmp (signal_spec, "object-signal::", 15) == 0)
 	sid = g_signal_connect_object (object, signal_spec + 15,
 				       callback, data,
 				       0);
-      else if (strncmp (signal_spec, "swapped_signal::", 16) == 0)
+      else if (strncmp (signal_spec, "swapped_signal::", 16) == 0 ||
+               strncmp (signal_spec, "swapped-signal::", 16) == 0)
 	sid = g_signal_connect_data (object, signal_spec + 16,
 				     callback, data, NULL,
 				     G_CONNECT_SWAPPED);
-      else if (strncmp (signal_spec, "swapped_object_signal::", 23) == 0)
+      else if (strncmp (signal_spec, "swapped_object_signal::", 23) == 0 ||
+               strncmp (signal_spec, "swapped-object-signal::", 23) == 0)
 	sid = g_signal_connect_object (object, signal_spec + 23,
 				       callback, data,
 				       G_CONNECT_SWAPPED);
-      else if (strncmp (signal_spec, "signal_after::", 14) == 0)
+      else if (strncmp (signal_spec, "signal_after::", 14) == 0 ||
+               strncmp (signal_spec, "signal-after::", 14) == 0)
 	sid = g_signal_connect_data (object, signal_spec + 14,
 				     callback, data, NULL,
 				     G_CONNECT_AFTER);
-      else if (strncmp (signal_spec, "object_signal_after::", 21) == 0)
+      else if (strncmp (signal_spec, "object_signal_after::", 21) == 0 ||
+               strncmp (signal_spec, "object-signal-after::", 21) == 0)
 	sid = g_signal_connect_object (object, signal_spec + 21,
 				       callback, data,
 				       G_CONNECT_AFTER);
-      else if (strncmp (signal_spec, "swapped_signal_after::", 22) == 0)
+      else if (strncmp (signal_spec, "swapped_signal_after::", 22) == 0 ||
+               strncmp (signal_spec, "swapped-signal-after::", 22) == 0)
 	sid = g_signal_connect_data (object, signal_spec + 22,
 				     callback, data, NULL,
 				     G_CONNECT_SWAPPED | G_CONNECT_AFTER);
-      else if (strncmp (signal_spec, "swapped_object_signal_after::", 29) == 0)
+      else if (strncmp (signal_spec, "swapped_object_signal_after::", 29) == 0 ||
+               strncmp (signal_spec, "swapped-object-signal-after::", 29) == 0)
 	sid = g_signal_connect_object (object, signal_spec + 29,
 				       callback, data,
 				       G_CONNECT_SWAPPED | G_CONNECT_AFTER);
       else
 	{
-	  g_warning ("%s: invalid signal spec \"%s\"", G_STRLOC, signal_spec);
+	  g_warning ("%s: invalid signal spec \"%s\"", G_STRFUNC, signal_spec);
 	  break;
 	}
       signal_spec = va_arg (var_args, gchar*);
@@ -1372,29 +1416,31 @@ g_object_disconnect (gpointer     _object,
       gpointer data = va_arg (var_args, gpointer);
       guint sid = 0, detail = 0, mask = 0;
 
-      if (strncmp (signal_spec, "any_signal::", 12) == 0)
+      if (strncmp (signal_spec, "any_signal::", 12) == 0 ||
+          strncmp (signal_spec, "any-signal::", 12) == 0)
 	{
 	  signal_spec += 12;
 	  mask = G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA;
 	}
-      else if (strcmp (signal_spec, "any_signal") == 0)
+      else if (strcmp (signal_spec, "any_signal") == 0 ||
+               strcmp (signal_spec, "any-signal") == 0)
 	{
 	  signal_spec += 10;
 	  mask = G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA;
 	}
       else
 	{
-	  g_warning ("%s: invalid signal spec \"%s\"", G_STRLOC, signal_spec);
+	  g_warning ("%s: invalid signal spec \"%s\"", G_STRFUNC, signal_spec);
 	  break;
 	}
 
       if ((mask & G_SIGNAL_MATCH_ID) &&
 	  !g_signal_parse_name (signal_spec, G_OBJECT_TYPE (object), &sid, &detail, FALSE))
-	g_warning ("%s: invalid signal name \"%s\"", G_STRLOC, signal_spec);
+	g_warning ("%s: invalid signal name \"%s\"", G_STRFUNC, signal_spec);
       else if (!g_signal_handlers_disconnect_matched (object, mask | (detail ? G_SIGNAL_MATCH_DETAIL : 0),
 						      sid, detail,
 						      NULL, (gpointer)callback, data))
-	g_warning (G_STRLOC ": signal handler %p(%p) is not connected", callback, data);
+	g_warning ("%s: signal handler %p(%p) is not connected", G_STRFUNC, callback, data);
       signal_spec = va_arg (var_args, gchar*);
     }
   va_end (var_args);
@@ -1481,7 +1527,7 @@ g_object_weak_unref (GObject    *object,
 	  }
     }
   if (!found_one)
-    g_warning (G_STRLOC ": couldn't find weak ref %p(%p)", notify, data);
+    g_warning ("%s: couldn't find weak ref %p(%p)", G_STRFUNC, notify, data);
 }
 
 void
